import 'dart:async';
import 'package:bytedesk_kefu/model/requestAnswer.dart';
import 'package:bytedesk_kefu/repositories/message_repository.dart';
import 'package:bloc/bloc.dart';
import './bloc.dart';

class MessageBloc extends Bloc<MessageEvent, MessageState> {
  //
  final MessageRepository messageRepository = new MessageRepository();

  MessageBloc() : super(InitialMessageState());

  @override
  Stream<MessageState> mapEventToState(MessageEvent event) async* {
    if (event is ReceiveMessageEvent) {
      yield* _mapRefreshCourseToState(event);
    } else if (event is UploadImageEvent) {
      yield* _mapUploadImageToState(event);
    } else if (event is QueryAnswerEvent) {
      yield* _mapQueryAnswerToState(event);
    } else if (event is MessageAnswerEvent) {
      yield* _mapMessageAnswerToState(event);
    } else if (event is RateAnswerEvent) {
      yield* _mapRateAnswerToState(event);
    }
  }

  Stream<MessageState> _mapRefreshCourseToState(
      ReceiveMessageEvent event) async* {
    try {
      yield ReceiveMessageState(message: event.message);
    } catch (error) {
      print(error);
    }
  }

  Stream<MessageState> _mapUploadImageToState(UploadImageEvent event) async* {
    yield MessageUpLoading();
    try {
      final String url = await messageRepository.upload(event.filePath);
      yield UploadImageSuccess(url);
    } catch (error) {
      print(error);
      yield UpLoadImageError();
    }
  }

  Stream<MessageState> _mapQueryAnswerToState(QueryAnswerEvent event) async* {
    yield MessageLoading();
    try {
      final RequestAnswerResult requestAnswerResult =
          await messageRepository.queryAnswer(event.tid, event.aid);
      yield QueryAnswerSuccess(
          query: requestAnswerResult.query, answer: requestAnswerResult.anwser);
    } catch (error) {
      print(error);
      yield UpLoadImageError();
    }
  }

  Stream<MessageState> _mapMessageAnswerToState(
      MessageAnswerEvent event) async* {
    yield MessageLoading();
    try {
      final RequestAnswerResult requestAnswerResult = await messageRepository
          .messageAnswer(event.type, event.wid, event.aid, event.content);
      yield MessageAnswerSuccess(
          query: requestAnswerResult.query, answer: requestAnswerResult.anwser);
    } catch (error) {
      print(error);
      yield UpLoadImageError();
    }
  }

  Stream<MessageState> _mapRateAnswerToState(RateAnswerEvent event) async* {
    yield MessageLoading();
    try {
      final RequestAnswerResult requestAnswerResult =
          await messageRepository.rateAnswer(event.aid, event.mid, event.rate);
      yield RateAnswerSuccess(result: requestAnswerResult.anwser);
    } catch (error) {
      print(error);
      yield UpLoadImageError();
    }
  }
}
