import 'dart:async';
import 'dart:io';

import 'package:bytedesk_kefu/http/bytedesk_device_api.dart';
import 'package:bytedesk_kefu/http/bytedesk_thread_api.dart';
import 'package:bytedesk_kefu/model/app.dart';
import 'package:bytedesk_kefu/model/thread.dart';
import 'package:bytedesk_kefu/ui/chat/page/chat_webview_page.dart';
import 'package:flutter/services.dart';

import 'package:bytedesk_kefu/http/bytedesk_user_api.dart';
import 'package:bytedesk_kefu/ui/chat/provider/chat_kf_provider.dart';
import 'package:bytedesk_kefu/util/bytedesk_utils.dart';
import 'package:bytedesk_kefu/util/bytedesk_constants.dart';
import 'package:flustars/flustars.dart';
import 'package:flutter/material.dart';

import 'model/user.dart';

class BytedeskKefu {
  static const MethodChannel _channel = const MethodChannel('bytedesk_kefu');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  // 下面为 自定义接口

  // 访客匿名登录接口
  static void anonymousLogin(
      String androidKey, String iOSKey, String subDomain) async {
    // 暂不需要，在chat_kf_page中有调用
    // new MessageProvider();
    String _appKey;
    if (BytedeskUtils.isAndroid) {
      // android
      _appKey = androidKey;
    } else if (BytedeskUtils.isIOS) {
      // ios
      _appKey = iOSKey;
    } else {
      // TODO: web/mac/windows
    }

    /// sp初始化
    await SpUtil.getInstance();
    // 首先检测是否是第一次，如果是第一此启动
    String username = SpUtil.getString(BytedeskConstants.username);
    // print('username:' + username);
    if (username.isEmpty) {
      // 则调用注册接口，否则调用登录接口
      User user = await BytedeskUserHttpApi().registerAnonymous(subDomain);
      //
      String password = user.username;
      login(user.username, password, _appKey, subDomain,
          BytedeskConstants.ROLE_VISITOR);
    } else {
      //
      String password = username;
      login(username, password, _appKey, subDomain,
          BytedeskConstants.ROLE_VISITOR);
    }
  }

  // 用户名登录
  static void login(String username, String password, String appkey,
      String subDomain, String role) async {
    //
    if (role == BytedeskConstants.ROLE_ADMIN) {
      if (!username.contains("@")) {
        username = username + "@" + subDomain;
      }
    }
    await BytedeskUserHttpApi().oauth(username, password);
    bool isLogin = SpUtil.getBool(BytedeskConstants.isLogin);
    if (isLogin) {
      // 登录成功之后，建立长连接
      BytedeskUtils.mqttConnect();
    }
    if (role == BytedeskConstants.ROLE_ADMIN) {
      // TODO: 如果是客服账号，加载个人信息
    }
    // 上传设备信息
    await BytedeskDeviceHttpApi().setDeviceInfo();
  }

  // TODO: 退出登录
  static void logout() {}

  // 建立长连接
  static bool connect() {
    return BytedeskUtils.mqttConnect();
  }

  // 判断长连接状态
  static bool isConnected() {
    return BytedeskUtils.isMqttConnected();
  }

  // 技能组客服会话
  static void startWorkGroupChat(
      BuildContext context, String wid, String title) {
    Navigator.of(context).push(new MaterialPageRoute(builder: (context) {
      return new ChatKFProvider(
        wid: wid,
        aid: '',
        type: BytedeskConstants.CHAT_TYPE_WORKGROUP,
        title: title,
      );
    }));
  }

  // 发送附言消息
  static void startWorkGroupChatPostscript(
      BuildContext context, String wid, String title, String postScript) {
    Navigator.of(context).push(new MaterialPageRoute(builder: (context) {
      return new ChatKFProvider(
        wid: wid,
        aid: '',
        type: BytedeskConstants.CHAT_TYPE_WORKGROUP,
        title: title,
        postscript: postScript,
      );
    }));
  }

  // 电商接口，携带商品参数
  // type/title/content/price/url/imageUrl/id/categoryCode
  static void startWorkGroupChatShop(
      BuildContext context, String wid, String title, String commodity) {
    Navigator.of(context).push(new MaterialPageRoute(builder: (context) {
      return new ChatKFProvider(
        wid: wid,
        aid: '',
        type: BytedeskConstants.CHAT_TYPE_WORKGROUP,
        title: title,
        custom: commodity,
      );
    }));
  }

  // 指定客服会话
  static void startAppointedChat(
      BuildContext context, String uid, String title) {
    Navigator.of(context).push(new MaterialPageRoute(builder: (context) {
      return new ChatKFProvider(
        wid: '',
        aid: uid,
        type: BytedeskConstants.CHAT_TYPE_APPOINTED,
        title: title,
      );
    }));
  }

  // 发送附言消息
  static void startAppointedChatPostscript(
      BuildContext context, String uid, String title, String postScript) {
    Navigator.of(context).push(new MaterialPageRoute(builder: (context) {
      return new ChatKFProvider(
        wid: '',
        aid: uid,
        type: BytedeskConstants.CHAT_TYPE_APPOINTED,
        title: title,
        postscript: postScript,
      );
    }));
  }

  // 电商接口，携带商品参数
  static void startAppointedChatShop(
      BuildContext context, String uid, String title, String commodity) {
    Navigator.of(context).push(new MaterialPageRoute(builder: (context) {
      return new ChatKFProvider(
        wid: '',
        aid: uid,
        type: BytedeskConstants.CHAT_TYPE_APPOINTED,
        title: title,
        custom: commodity,
      );
    }));
  }

  static void startH5Chat(BuildContext context, String url, String title) {
    Navigator.of(context).push(new MaterialPageRoute(builder: (context) {
      return new ChatWebViewPage(
        url: url,
        title: title,
      );
    }));
  }

  // 获取个人资料
  static Future<User> getProfile() async {
    return BytedeskUserHttpApi().getProfile();
  }

  // 设置用户昵称
  static Future<User> updateNickname(String nickname) async {
    return BytedeskUserHttpApi().updateNickname(nickname);
  }

  // 设置用户头像
  static Future<User> updateAvatar(String avatar) async {
    return BytedeskUserHttpApi().updateAvatar(avatar);
  }

  // 查询技能组在线状态
  static Future<String> getWorkGroupStatus(String workGroupWid) async {
    return BytedeskUserHttpApi().getWorkGroupStatus(workGroupWid);
  }

  // 查询客服在线状态
  static Future<String> getAgentStatus(String agentUid) async {
    return BytedeskUserHttpApi().getAgentStatus(agentUid);
  }

  // 访客历史会话
  static Future<List<Thread>> getVisitorThreads(int page, int size) async {
    return BytedeskThreadHttpApi().getVisitorThreads(page, size);
  }

  // 消息提示设置
  static bool getPlayAudioOnSendMessage() {
    return SpUtil.getBool(BytedeskConstants.PLAY_AUDIO_ON_SEND_MESSAGE);
  }

  static void setPlayAudioOnSendMessage(bool flag) {
    SpUtil.putBool(BytedeskConstants.PLAY_AUDIO_ON_SEND_MESSAGE, flag);
  }

  static bool getPlayAudioOnReceiveMessage() {
    return SpUtil.getBool(BytedeskConstants.PLAY_AUDIO_ON_RECEIVE_MESSAGE);
  }

  static void setPlayAudioOnReceiveMessage(bool flag) {
    SpUtil.putBool(BytedeskConstants.PLAY_AUDIO_ON_RECEIVE_MESSAGE, flag);
  }

  static bool getVibrateOnReceiveMessage() {
    return SpUtil.getBool(BytedeskConstants.VIBRATE_ON_RECEIVE_MESSAGE);
  }

  static void setVibrateOnReceiveMessage(bool flag) {
    SpUtil.putBool(BytedeskConstants.VIBRATE_ON_RECEIVE_MESSAGE, flag);
  }

  // build的值只有两种情况，测试设置为：debug 或 线上设置为：release
  Future<void> uploadIOSDeviceToken(
      String appkey, String build, String deviceToken) async {
    BytedeskDeviceHttpApi().updateIOSDeviceToken(appkey, build, deviceToken);
  }

  // build的值只有两种情况，测试设置为：debug 或 线上设置为：release
  Future<void> deleteIOSDeviceToken(String build) async {
    BytedeskDeviceHttpApi().deleteIOSDeviceToken(build);
  }

  // 获取当前APP版本
  static Future<String> getAppVersion() {
    return BytedeskDeviceHttpApi().getAppVersion();
  }

  // 从服务器检测当前APP是否有新版
  static Future<App> checkAppVersion(String androidKey, String iosKey) {
    if (Platform.isAndroid) {
      return BytedeskUserHttpApi().checkAppVersion(androidKey);
    }
    return BytedeskUserHttpApi().checkAppVersion(iosKey);
  }

}
