import 'dart:async';
// In order to *not* need this ignore, consider extracting the "web" version
// of your plugin as a separate package, instead of inlining it in the same
// package as the core of your plugin.
// ignore: avoid_web_libraries_in_flutter
import 'dart:html' as html show window;

import 'package:flutter/services.dart';
import 'package:flutter_web_plugins/flutter_web_plugins.dart';

import 'package:bytedesk_kefu/http/bytedesk_user_api.dart';
import 'package:bytedesk_kefu/ui/chat/provider/chat_kf_provider.dart';
import 'package:bytedesk_kefu/util/bytedesk_utils.dart';
import 'package:bytedesk_kefu/util/bytedesk_constants.dart';
import 'package:flustars/flustars.dart';
import 'package:flutter/material.dart';

import 'model/user.dart';

/// A web implementation of the BytedeskKefu plugin.
class BytedeskKefuWeb {
  static void registerWith(Registrar registrar) {
    final MethodChannel channel = MethodChannel(
      'bytedesk_kefu',
      const StandardMethodCodec(),
      registrar.messenger,
    );

    final pluginInstance = BytedeskKefuWeb();
    channel.setMethodCallHandler(pluginInstance.handleMethodCall);
  }

  /// Handles method calls over the MethodChannel of this plugin.
  /// Note: Check the "federated" architecture for a new way of doing this:
  /// https://flutter.dev/go/federated-plugins
  Future<dynamic> handleMethodCall(MethodCall call) async {
    switch (call.method) {
      case 'getPlatformVersion':
        return getPlatformVersion();
        break;
      default:
        throw PlatformException(
          code: 'Unimplemented',
          details: 'bytedesk_kefu for web doesn\'t implement \'${call.method}\'',
        );
    }
  }

  /// Returns a [String] containing the version of the platform.
  Future<String> getPlatformVersion() {
    final version = html.window.navigator.userAgent;
    return Future.value(version);
  }

  // 自定义接口

  // 访客匿名登录接口
  static void anonymousLogin(String androidKey, String iOSKey, String subDomain) async {
    // 暂不需要，在chat_kf_page中有调用
    // new MessageProvider();
     String _appKey;
    if (BytedeskUtils.isAndroid) {
      // android
      _appKey = androidKey;
    } else {
      // ios
      _appKey = iOSKey;
    }
    /// sp初始化
    await SpUtil.getInstance();
    // 首先检测是否是第一次，如果是第一此启动
    String username = SpUtil.getString(BytedeskConstants.username);
    // print('username:' + username);
    if (username.isEmpty) {
      // 则调用注册接口，否则调用登录接口
      User user = await BytedeskUserHttpApi().registerAnonymous(subDomain);
      //
      String password = user.username;
      login(user.username, password, _appKey, subDomain,
          BytedeskConstants.ROLE_VISITOR);
    } else {
      //
      String password = username;
      login(username, password, _appKey, subDomain,
          BytedeskConstants.ROLE_VISITOR);
    }
  }

  static void login(String username, String password, String appkey,
      String subDomain, String role) async {
    //
    if (role == BytedeskConstants.ROLE_ADMIN) {
      if (!username.contains("@")) {
        username = username + "@" + subDomain;
      }
    }
    await BytedeskUserHttpApi().oauth(username, password);
    bool isLogin = SpUtil.getBool(BytedeskConstants.isLogin);
    if (isLogin) {
      // 登录成功之后，建立长连接
      BytedeskUtils.mqttConnect();
    }
  }

  // 建立长连接
  static bool connect() {
    return BytedeskUtils.mqttConnect();
  }

  // 技能组客服会话
  static void startWorkGroupChat(
      BuildContext context, String wid, String title) {
    Navigator.of(context).push(new MaterialPageRoute(builder: (context) {
      return new ChatKFProvider(
        wid: wid,
        aid: '',
        type: BytedeskConstants.CHAT_TYPE_WORKGROUP,
        title: title,
      );
    }));
  }

  // 发送附言消息
  static void startWorkGroupChatPostscript(
      BuildContext context, String wid, String title, String postScript) {
    Navigator.of(context).push(new MaterialPageRoute(builder: (context) {
      return new ChatKFProvider(
        wid: wid,
        aid: '',
        type: BytedeskConstants.CHAT_TYPE_WORKGROUP,
        title: title,
      );
    }));
  }

  // 电商接口，携带商品参数
  // type/title/content/price/url/imageUrl/id/categoryCode
  static void startWorkGroupChatShop(BuildContext context, String wid, String title, String commodity) {
    Navigator.of(context).push(new MaterialPageRoute(builder: (context) {
      return new ChatKFProvider(
        wid: wid,
        aid: '',
        type: BytedeskConstants.CHAT_TYPE_WORKGROUP,
        title: title,
        custom: commodity,
      );
    }));
  }

  // 指定客服会话
  static void startAppointedChat(
      BuildContext context, String uid, String title) {
    Navigator.of(context).push(new MaterialPageRoute(builder: (context) {
      return new ChatKFProvider(
        wid: '',
        aid: uid,
        type: BytedeskConstants.CHAT_TYPE_APPOINTED,
        title: title,
      );
    }));
  }

  // 发送附言消息
  static void startAppointedChatPostscript(
      BuildContext context, String uid, String title, String postScript) {
    Navigator.of(context).push(new MaterialPageRoute(builder: (context) {
      return new ChatKFProvider(
        wid: '',
        aid: uid,
        type: BytedeskConstants.CHAT_TYPE_APPOINTED,
        title: title,
      );
    }));
  }

  // 电商接口，携带商品参数
  static void startAppointedChatShop(BuildContext context, String uid, String title, String commodity) {
    Navigator.of(context).push(new MaterialPageRoute(builder: (context) {
      return new ChatKFProvider(
        wid: '',
        aid: uid,
        type: BytedeskConstants.CHAT_TYPE_APPOINTED,
        title: title,
        custom: commodity,
      );
    }));
  }

}
