import 'dart:convert';
import 'dart:io';

import 'package:bytedesk_kefu/model/requestAnswer.dart';
import 'package:bytedesk_kefu/util/bytedesk_constants.dart';
import 'package:flustars/flustars.dart';
import 'package:http/http.dart' as http;

class BytedeskMessageHttpApi {
  //
  static const uid = BytedeskConstants.HELP_CATEGORY_UID;
  static const client = BytedeskConstants.client;
  static String baseUrl = Platform.isAndroid
      ? BytedeskConstants.httpBaseUrlAndroid
      : BytedeskConstants.httpBaseUrliOS;

  final http.Client httpClient = http.Client();

  BytedeskMessageHttpApi();

  //
  Future<RequestAnswerResult> queryAnswer(String tid, String aid) async {
    //
    String accessToken = SpUtil.getString(BytedeskConstants.accessToken);
    Map<String, String> headers = {
      // "Accept":  "application/json",
      HttpHeaders.contentTypeHeader: "application/json",
      HttpHeaders.authorizationHeader: "Bearer $accessToken"
    };
    //
    final queryAnswerUrl =
        '$baseUrl/api/answer/query?tid=$tid&aid=$aid&client=$client';
    print("query Url $queryAnswerUrl");
    final initResponse =
        await this.httpClient.get(queryAnswerUrl, headers: headers);
    //
    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    //
    return RequestAnswerResult.fromJson(responseJson);
  }

  //
  Future<RequestAnswerResult> messageAnswer(
      String type, String wid, String aid, String content) async {
    //
    String accessToken = SpUtil.getString(BytedeskConstants.accessToken);
    Map<String, String> headers = {
      HttpHeaders.contentTypeHeader: "application/json",
      HttpHeaders.authorizationHeader: "Bearer $accessToken"
    };
    //
    final messageAnswerUrl =
        '$baseUrl/api/v2/answer/message?type=$type&wid=$wid&aid=$aid&content=$content&client=$client';
    print("message Url $messageAnswerUrl");
    final initResponse =
        await this.httpClient.get(messageAnswerUrl, headers: headers);
    //
    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    print("messageAnswer responseJson $responseJson");
    //
    return RequestAnswerResult.fromJson(responseJson);
  }

  //
  Future<RequestAnswerResult> rateAnswer(
      String aid, String mid, bool rate) async {
    //
    String accessToken = SpUtil.getString(BytedeskConstants.accessToken);
    Map<String, String> headers = {
      HttpHeaders.contentTypeHeader: "application/json",
      HttpHeaders.authorizationHeader: "Bearer $accessToken"
    };
    //
    final rateAnswerUrl =
        '$baseUrl/api/answer/rate?aid=$aid&mid=$mid&rate=$rate&client=$client';
    print("rate Url $rateAnswerUrl");
    final initResponse =
        await this.httpClient.get(rateAnswerUrl, headers: headers);
    //
    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson =
        json.decode(utf8decoder.convert(initResponse.bodyBytes));
    //
    return RequestAnswerResult.fromRateJson(responseJson);
  }

  // https://pub.dev/documentation/http/latest/http/MultipartRequest-class.html
  Future<String> upload(String filePath) async {
    //
    String fileName = filePath.split("/").last;
    String username = SpUtil.getString(BytedeskConstants.uid);

    final uploadUrl = '$baseUrl/visitor/api/upload/image';
    print("fileName $fileName, username $username, upload Url $uploadUrl");

    var uri = Uri.parse(uploadUrl);
    var request = http.MultipartRequest('POST', uri)
      ..fields['file_name'] = fileName
      ..fields['username'] = username
      ..files.add(await http.MultipartFile.fromPath('file', filePath));

    http.Response response =
        await http.Response.fromStream(await request.send());
    // print("Result: ${response.body}");

    //解决json解析中的乱码问题
    Utf8Decoder utf8decoder = Utf8Decoder(); // fix 中文乱码
    //将string类型数据 转换为json类型的数据
    final responseJson = json.decode(utf8decoder.convert(response.bodyBytes));
    print("responseJson $responseJson");

    String url = responseJson['data'];
    print('url:' + url);
    return url;
  }

}
