import 'dart:async';

import 'package:path/path.dart';
import 'package:sqflite/sqflite.dart';
import 'package:bytedesk_kefu/model/model.dart';

// https://pub.dev/packages/sqflite
class MessageProvider {
  //
  static final MessageProvider _singleton = MessageProvider._internal();
  factory MessageProvider() {
    return _singleton;
  }
  MessageProvider._internal() {
    open();
  }
  //
  final String tableMessage = 'messages';
  final String columnId = '_id';
  final String columnMid = 'mid';
  final String columnType = 'type';
  final String columnTopic = 'topic';
  final String columnContent = 'content';
  final String columnImageUrl = 'imageUrl';
  final String columnAvatar = 'avatar';
  final String columnStatus = 'status';
  final String columnIsSend = 'isSend';
  final String columnTimestamp = 'timestamp';
  final String columnCurrentUid = 'currentUid';
  //
  Database database;

  Future open() async {
    // Open the database and store the reference.
    database = await openDatabase(
      // Set the path to the database. Note: Using the `join` function from the
      // `path` package is best practice to ensure the path is correctly
      // constructed for each platform.
      join(await getDatabasesPath(), 'bytedeskv4.db'),
      // When the database is first created, create a table to store dogs.
      onCreate: (db, version) {
        // Run the CREATE TABLE statement on the database.
        return db.execute(
          "CREATE TABLE $tableMessage($columnId INTEGER PRIMARY KEY autoincrement, " +
              "$columnMid TEXT, $columnType TEXT, $columnTopic TEXT, $columnContent TEXT, $columnImageUrl TEXT, $columnAvatar TEXT, $columnStatus TEXT, " +
              "$columnIsSend integer, $columnTimestamp TEXT, $columnCurrentUid TEXT)",
        );
      },
      // Set the version. This executes the onCreate function and provides a
      // path to perform database upgrades and downgrades.
      version: 4,
    );
  }

  Future<int> insert(Message message) async {
    // print('insert avatar:' + message.avatar + ' conten:' + message.content + ' timestamp:' + message.timestamp);
    return await database.insert(tableMessage, message.toMap());
  }

  //
  Future<List<Message>> getTopicMessages(
      String topic, String currentUid, int page, int size) async {
    //
    List<Map> maps = await database.query(tableMessage,
        columns: [
          columnMid,
          columnContent,
          columnImageUrl,
          columnType,
          columnTopic,
          columnStatus,
          columnTimestamp,
          columnAvatar,
          columnIsSend
        ],
        where: '$columnTopic = ? and $columnCurrentUid = ?',
        whereArgs: [topic, currentUid],
        orderBy: '$columnTimestamp DESC',
        limit: size,
        offset: page * size);
    //
    return List.generate(maps.length, (i) {
      return Message(
          mid: maps[i]['mid'],
          content: maps[i]['content'],
          imageUrl: maps[i]['imageUrl'],
          type: maps[i]['type'],
          topic: maps[i]['topic'],
          status: maps[i]['status'],
          timestamp: maps[i]['timestamp'],
          avatar: maps[i]['avatar'],
          isSend: maps[i]['isSend'],
          answers: []);
    });
  }

  Future<int> delete(String mid) async {
    return await database
        .delete(tableMessage, where: '$columnMid = ?', whereArgs: [mid]);
  }

  Future<int> update(String mid, String status) async {
    return await database.rawUpdate(
        'UPDATE $tableMessage SET $columnStatus = ? WHERE $columnMid = ?',
        [status, mid]);
  }

  Future close() async => database.close();
}

// Future<Message> getMessage(String mid) async {
//   List<Map> maps = await database.query(tableMessage,
//       columns: [
//         columnMid,
//         columnContent,
//         columnImageUrl,
//         columnType,
//         columnTopic
//       ],
//       where: '$columnMid = ?',
//       whereArgs: [mid]);
//   if (maps.length > 0) {
//     return Message.fromMap(maps.first);
//   }
//   return null;
// }
// Future<List<Message>> getMessages() async {
//   // Query the table for all The Dogs.
//   final List<Map<String, dynamic>> maps = await database.query(tableMessage);

//   // Convert the List<Map<String, dynamic> into a List<Dog>.
//   return List.generate(maps.length, (i) {
//     return Message(
//         mid: maps[i]['mid'],
//         content: maps[i]['content'],
//         imageUrl: maps[i]['imageUrl'],
//         type: maps[i]['type'],
//         topic: maps[i]['topic'],
//         timestamp: maps[i]['timestamp'],
//         avatar: maps[i]['avatar'],
//         isSend: maps[i]['isSend']);
//   });
// }
// Future<int> update(Message message) async {
//   return await database.update(tableMessage, message.toMap(),
//       where: '$columnMid = ?', whereArgs: [message.mid]);
// }
